@@ -1,8 +1,9 @@
 package Term::ProgressBar;
+
 use strict;
 use warnings;
 
-our $VERSION = '2.16';
+our $VERSION = '2.22';
 
 #XXX TODO Redo original test with count=20
 #         Amount Output
@@ -19,12 +20,16 @@ our $VERSION = '2.16';
 
 Term::ProgressBar - provide a progress meter on a standard terminal
 
+=head1 VERSION
+
+Version 2.22
+
 =head1 SYNOPSIS
 
-  use Term::ProgressBar;
+    use Term::ProgressBar;
 
-  $progress = Term::ProgressBar->new ({count => $count});
-  $progress->update ($so_far);
+    my $progress = Term::ProgressBar->new ({count => 10_000});
+    $progress->update(5_000);
 
 =head1 DESCRIPTION
 
@@ -57,26 +62,24 @@ bar for the same width.
 
 =head2 A really simple use
 
-  #!/usr/bin/perl
+    #!/usr/bin/perl
 
-  use Term::ProgressBar 2.00;
+    use Term::ProgressBar 2.00;
+    use constant MAX => 100_000;
 
-  use constant MAX => 100_000;
+    my $progress = Term::ProgressBar->new(MAX);
 
-  my $progress = Term::ProgressBar->new(MAX);
+    for (0..MAX) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
 
-  for (0..MAX) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+        if ($is_power) {
+            $progress->update($_);
+        }
     }
 
-    if ( $is_power ) {
-      $progress->update($_);
-    }
-  }
-
 see eg/simle_use.pl
 
 Here is a simple example.  The process considers all the numbers between 0 and
@@ -89,18 +92,17 @@ distribution set (it is not installed as part of the module).
 
 =head2 A smoother bar update
 
-  my $progress = Term::ProgressBar->new($max);
+    my $progress = Term::ProgressBar->new($max);
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
+
+        $progress->update($_)
     }
 
-    $progress->update($_)
-  }
-
 See eg/smooth_bar.pl
 
 This example calls update for each value considered.  This will result in a
@@ -115,22 +117,20 @@ distribution set (it is not installed as part of the module.
 
 =head2 A (much) more efficient update
 
-  my $progress = Term::ProgressBar->new({name => 'Powers', count => $max, remove => 1});
-  $progress->minor(0);
-  my $next_update = 0;
+    my $progress = Term::ProgressBar->new({name => 'Powers', count => $max, remove => 1});
+    $progress->minor(0);
+    my $next_update = 0;
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      $is_power = 1
-        if 2**$i == $_;
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+            $is_power = 1 if 2**$i == $_;
+        }
+
+        $next_update = $progress->update($_) if $_ >= $next_update;
     }
 
-    $next_update = $progress->update($_)
-      if $_ >= $next_update;
-  }
-  $progress->update($max)
-    if $max >= $next_update;
+    $progress->update($max) if $max >= $next_update;
 
 This example does two things to improve efficiency: firstly, it uses the value
 returned by L<update|"update"> to only call it again when needed; secondly, it
@@ -147,21 +147,84 @@ progress bar from the terminal when done.
 The complete text of this example is in F<examples/powers3> in the
 distribution set (it is not installed as part of the module.
 
+=head2 When the maximum number of items is sometimes unknown
+
+Sometimes you may wish to use the progress bar when the number of items may or
+may not be known. One common example is when you write a script that can take
+input piped from the output of another command, and then pipe the output to yet
+another command. eg:
+
+  some_command --arg value | my_script.pl | some_other_command
+
+Or ...
+
+  my_script.pl input_file output_file
+
+This example shows how you can iterate over a file specified on the command line
+with the progress bar. Since the input file may be read from STDIN, the number
+of lines may not be known. Term::ProgressBar handles this by just taking '-1' as
+the count value and with no further changes to the code. By calling update
+with the same count value, you ensure the progress bar is removed afterwards.
+
+    my $input_file = shift;
+    my $output_file = shift;
+    my $in_fh = \*STDIN;
+    my $out_fh = \*STDOUT;
+    my $message_fh = \*STDERR;
+    my $num_lines = -1;
+
+    if (defined($input_file) and $input_file ne '-') {
+        open($in_fh, $input_file) or die "Couldn't open file, '$input_file': $!";
+        my $wc_output = `wc -l $input_file`;
+        chomp($wc_output);
+        $wc_output =~ /^\s*(\d+)(\D.*)?/ or die "Couldn't parse wc output: $wc_output";
+        $num_lines = $1;
+    }
+
+    if(defined($output_file)) {
+        !-f $output_file or die "Specified output file, '$output_file', already exists";
+        open($out_fh, '>', $output_file) or die "Couldn't open output file, '$output_file': $!";
+    }
+
+    my $progress = Term::ProgressBar->new({
+        name   => 'file processor',
+        count  => $num_lines,
+        remove => 1,
+        fh     => $message_fh,
+    });
+
+    while (my $line = <$in_fh>) {
+        chomp($line);
+        print $out_fh "I found a line: $line\n";
+        $progress->message("Found 10000!") if($line =~ /10000/);
+        $progress->update();
+    }
+
+    $progress->update($num_lines);
+
+    print $message_fh "Finished\n";
+
+When the file is defined explicitly, the progress bar displays the linewise
+progress through the file. Since the progress bar by default prints output to
+stderr, your scripts output to STDOUT will not be affected.
+
 =head2 Using Completion Time Estimation
 
-  my $progress = Term::ProgressBar->new({name  => 'Powers',
-                                         count => $max,
-                                         ETA   => 'linear', });
-  $progress->max_update_rate(1);
-  my $next_update = 0;
+    my $progress = Term::ProgressBar->new({
+        name  => 'Powers',
+        count => $max,
+        ETA   => 'linear',
+    });
+    $progress->max_update_rate(1);
+    my $next_update = 0;
 
-  for (0..$max) {
-    my $is_power = 0;
-    for(my $i = 0; 2**$i <= $_; $i++) {
-      if ( 2**$i == $_ ) {
-        $is_power = 1;
-        $progress->message(sprintf "Found %8d to be 2 ** %2d", $_, $i);
-      }
+    for (0..$max) {
+        my $is_power = 0;
+        for (my $i = 0; 2**$i <= $_; $i++) {
+        if ( 2**$i == $_ ) {
+            $is_power = 1;
+            $progress->message(sprintf "Found %8d to be 2 ** %2d", $_, $i);
+        }
     }
 
     $next_update = $progress->update($_)
@@ -300,6 +363,12 @@ a number, being equivalent to the C<count> key.
 The item count.  The progress is marked at 100% when update I<count> is
 invoked, and proportionally until then.
 
+If you specify a count less than zero, just the name (if specified) will be
+displayed and (if the remove flag is set) removed when the progress bar is
+updated with a number lower than zero. This allows you to use the progress bar
+when the count is sometimes known and sometimes not without making multiple
+changes throughout your code.
+
 =item name
 
 A name to prefix the progress bar with.
@@ -394,7 +463,6 @@ Class::MethodMaker->import (new_with_init => 'new',
 
 sub init {
   my $self = shift;
-  return if $self->silent;
 
   # V1 Compatibility
   return $self->init({count      => $_[1], name => $_[0],
@@ -615,7 +683,6 @@ Class::MethodMaker->import
 # We generate these by hand since we want to check the values.
 sub bar_width {
     my $self = shift;
-    return if $self->silent;
     return $self->{bar_width} if not @_;
     croak 'wrong number of arguments' if @_ != 1;
     croak 'bar_width < 1' if $_[0] < 1;
@@ -623,7 +690,6 @@ sub bar_width {
 }
 sub term_width {
     my $self = shift;
-    return if $self->silent;
     return $self->{term_width} if not @_;
     croak 'wrong number of arguments' if @_ != 1;
     croak 'term_width must be at least 5' if $self->term and $_[0] < 5;
@@ -632,7 +698,6 @@ sub term_width {
 
 sub target {
   my $self = shift;
-  return if $self->silent;
 
   if ( @_ ) {
     my ($target) = @_;
@@ -719,7 +784,10 @@ The next value of so_far at which to call C<update>.
 
 sub update {
   my $self = shift;
-  return if $self->silent;
+  # returning target+1 as next value should avoid calling update
+  # method in the smooth form of using the progress bar
+  return $self->target+1 if $self->silent;
+
   my ($so_far) = @_;
 
   if ( ! defined $so_far ) {
@@ -735,12 +803,28 @@ sub update {
   my $name = $self->name;
   my $fh = $self->fh;
 
-  if ( $target < 1 ) {
+
+  if ( $target < 0 ) {
+    if($input_so_far <= 0 or $input_so_far == $self->last_update) {
+      print $fh "\r", ' ' x $self->term_width, "\r";
+
+      if(defined $name) {
+        if(!$self->remove or $input_so_far >= 0) {
+          print $fh "$name...";
+        }
+		if(!$self->remove and $input_so_far < 0) {
+		  print $fh "\n";
+        }
+      }
+    }
+    $self->last_update($input_so_far);
+    return 2**32-1;
+  } elsif ( $target == 0 ) {
     print $fh "\r";
     printf $fh "$name: "
       if defined $name;
     print $fh "(nothing to do)\n";
-    return 2**32-1;
+	return 2**32-1;
   }
 
   my $biggies     = $self->major_units * $so_far;
@@ -850,7 +934,7 @@ sub update {
        unless $self->scale == 1;
     } else {
       my $pc = int(100*$input_so_far/$target);
-      printf $fh "[%s] %s: %3d%%\n", scalar(localtime), $name, $pc;
+      printf $fh "[%s] %s: %3d%%\n", scalar(localtime), ($name || ''), $pc;
 
       $next = ceil($target * ($pc+1)/100);
     }
@@ -941,7 +1025,7 @@ Significant contributions from Ed Avis, amongst others.
 
 Gabor Szabo L<http://szabgab.com/> L<http://perlmaven.com/>
 
-=head1 COPYRIGHT
+=head1 LICENSE AND COPYRIGHT
 
 Copyright (c) 2001, 2002, 2003, 2004, 2005 Martyn J. Pearce.  This program is
 free software; you can redistribute it and/or modify it under the same terms
@@ -949,6 +1033,6 @@ as Perl itself.
 
 =cut
 
-1; # keep require happy.
+1;
 
 __END__
